class CandlestickInterval:
    MIN1 = "1min"
    MIN5 = "5min"
    MIN15 = "15min"
    MIN30 = "30min"
    MIN60 = "60min"
    HOUR4 = "4hour"
    DAY1 = "1day"
    MON1 = "1mon"
    WEEK1 = "1week"
    YEAR1 = "1year"
    INVALID = None


class OrderSide:
    BUY = "buy"
    SELL = "sell"
    INVALID = None


class TradeDirection:
    BUY = "buy"
    SELL = "sell"
    INVALID = None


class OrderType:
    SELL_LIMIT = "sell-limit"
    BUY_LIMIT = "buy-limit"
    BUY_MARKET = "buy-market"
    SELL_MARKET = "sell-market"
    BUY_IOC = "buy-ioc"
    SELL_IOC = "sell-ioc"
    BUY_LIMIT_MAKER = "buy-limit-maker"
    SELL_LIMIT_MAKER = "sell-limit-maker"
    BUY_STOP_LIMIT = "buy-stop-limit"
    SELL_STOP_LIMIT = "sell-stop-limit"
    BUY_LIMIT_FOK = "buy-limit-fok"
    SELL_LIMIT_FOK = "sell-limit-fok"
    BUY_STOP_LIMIT_FOK = "buy-stop-limit-fok"
    SELL_STOP_LIMIT_FOK = "sell-stop-limit-fok"
    INVALID = None


class AlgoOrderType:
    LIMIT = "limit"
    MARKET = "market"


class AlgoOrderStatus:
    CANCELED = "canceled"
    REJECTED = "rejected"
    TRIGGERED = "triggered"


class AccountType:
    SPOT = "spot"
    MARGIN = "margin"
    OTC = "otc"
    POINT = "point"
    MINEPOLL = "minepool"
    ETF = "etf"
    AGENCY = "agency"
    SUPER_MARGIN = "super-margin"
    INVALID = None


class AccountState:
    WORKING = "working"
    LOCK = "lock"
    INVALID = None


class AccountPointState:
    WORKING = "working"
    LOCK = "lock"
    INVALID = None
    FL_SYS = "fl-sys"
    FL_MGT = "fl-mgt"
    FL_END = "fl-end"
    FL_NEGATIVE = "fl-negative"


class AccountBalanceUpdateType:
    TRADE = "trade"
    FROZEN = "frozen"
    LOAN = "loan"
    INTEREST = "interest"
    LOAN_AVAILABLE = "loan-available"
    TRANSFER_OUT_AVAILABLE = "transfer-out-available"
    INVALID = None


class WithdrawState:
    SUBMITTED = "submitted"
    REEXAMINE = "reexamine"
    CANCELED = "canceled"
    PASS = "pass"
    REJECT = "reject"
    PRETRANSFER = "pre-transfer"
    WALLETTRANSFER = "wallet-transfer"
    WALEETREJECT = "wallet-reject"
    CONFIRMED = "confirmed"
    CONFIRMERROR = "confirm-error"
    REPEALED = "repealed"
    VERIFYING = "verifying"
    FAILED = "failed"
    INVALID = None


class DepositWithdraw:
    DEPOSIT = "deposit"
    WITHDRAW = "withdraw"


class DepositState:
    CONFIRMING = "confirming"
    SAFE = "safe"
    CONFIRMED = "confirmed"
    ORPHAN = "orphan"
    INVALID = None


class LoanOrderState:
    CREATED = "created"
    ACCRUAL = "accrual"
    CLEARED = "cleared"
    FAILED = "failed"
    INVALID = None


class OrderSource:
    SYS = "sys"
    WEB = "web"
    API = "api"
    APP = "app"
    FL_SYS = "fl-sys"
    FL_MGT = "fl-mgt"
    SPOT_WEB = "spot-web"
    SPOT_API = "spot-api"
    SPOT_APP = "spot-app"
    MARGIN_API = "margin-api"
    MARGIN_WEB = "margin-web"
    MARGIN_APP = "margin-app"
    SUPER_MARGIN_API = "super-margin-api"
    SUPER_MARGIN_WEB = "super-margin-web"
    SUPER_MARGIN_APP = "super-margin-app"
    SUPER_MARGIN_FL_SYS = "super-margin-fl-sys"
    SUPER_MARGIN_FL_MGT = "super-margin-fl-mgt"
    INVALID = None


class OrderState:
    CREATED = "created"  # for stop loss order
    PRE_SUBMITTED = "pre-submitted"
    SUBMITTING = "submitting"
    SUBMITTED = "submitted"
    PARTIAL_FILLED = "partial-filled"
    CANCELLING = "cancelling"
    PARTIAL_CANCELED = "partial-canceled"
    FILLED = "filled"
    CANCELED = "canceled"
    FAILED = "failed"
    PLACE_TIMEOUT = "place_timeout"
    INVALID = None


class TimeInForceType:
    IOC = "ioc"
    FOK = "fok"
    BOC = "boc"
    GTC = "gtc"


class TransferMasterType:
    IN = "master-transfer-in"
    OUT = "master-transfer-out"
    POINT_IN = "master-point-transfer-in"
    POINT_OUT = "master-point-transfer-out"
    INVALID = None


class EtfStatus:
    NORMAL = "1"
    REBALANCING_START = "2"
    CREATION_AND_REDEMPTION_SUSPEND = "3"
    CREATION_SUSPEND = "4"
    REDEMPTION_SUSPEND = "5"
    INVALID = None


class EtfSwapType:
    IN = "1"
    OUT = "2"
    INVALID = None


class BalanceMode:
    AVAILABLE = "0"
    TOTAL = "1"
    INVALID = None


class AccountBalanceMode:
    BALANCE = "0"
    TOTAL = "1"
    INVALID = None


class OperateMode:
    PING = "ping"
    PONG = "pong"
    INVALID = None


class QueryDirection:
    PREV = "prev"
    NEXT = "next"
    INVALID = None


class TransferFuturesPro:
    TO_PRO = "futures-to-pro"
    TO_FUTURES = "pro-to-futures"


class MatchRole:
    MAKER = "maker"
    TAKER = "taker"


class DepthStep:
    STEP0 = "step0"
    STEP1 = "step1"
    STEP2 = "step2"
    STEP3 = "step3"
    STEP4 = "step4"
    STEP5 = "step5"


class DepthSize:
    SIZE5 = 5
    SIZE10 = 10
    SIZE20 = 20


class MbpLevel:
    MBP5 = 5
    MBP10 = 10
    MBP20 = 20
    MBP150 = 150


class ChainDepositStatus:
    ALLOWED = "allowed"
    PROHIBITED = "prohibited"
    INVALID = None


class ChainWithdrawStatus:
    ALLOWED = "allowed"
    PROHIBITED = "prohibited"
    INVALID = None


class InstrumentStatus:
    NORMAL = "normal"
    DELISTED = "delisted"
    INVALID = None


class AccountChangeType:
    ORDER_PLACE = "order-place"
    ORDER_MATCH = "order-match"
    ORDER_REFUND = "order-refund"
    ORDER_CANCEL = "order-cancel"
    ORDER_FEE_REFUND = "order-fee-refund"
    MARGIN_TRANSFER = "margin-transfer"
    MARGIN_LOAN = "margin-loan"
    MARGIN_INTEREST = "margin-interest"
    MARGIN_REPAY = "margin-repay"
    OTHER = "other"
    DEPOSIT = "deposit"
    WITHDRAW = "withdraw"
    INVALID = None


class FeeDeductType:
    DEDUCT_BY_HT = "ht"
    DEDUCT_BY_POINT = "point"
    INVALID = None


class SubUidAction:
    UNLOCK = "unlock"
    LOCK = "lock"
    INVALID = None


class SubUidState:
    NORMAL = "normal"
    LOCK = "lock"
    INVALID = None


class OrderUpdateEventType:
    CREATION = "creation"
    TRADE = "trade"
    CANCELLATION = "cancellation"
    INVALID = None


class AccountTransactType:
    TRADE = "trade"
    ETF = "etf"
    TRANSACT_FEE = "transact-fee"
    DEDUCTION = "deduction"
    TRANSFER = "transfer"
    CREDIT = "credit"
    LIQUIDATION = "liquidation"
    INTEREST = "interest"
    DEPOSIT = "deposit"
    WITHDRAW = "withdraw"
    WITHDRAW_FEE = "withdraw-fee"
    EXCHANGE = "exchange"
    OTHER = "other-types"


class SortDesc:
    ASC = "asc"
    DESC = "desc"


class SubuserTradePrivilegeType:
    MARGIN = "isolated-margin"
    SUPER_MARGIN = "cross-margin"


class SubUserTradeStatus:
    ACTIVATED = "activated"
    DEACTIVATED = "deactivated"


class MarketStatus:
    NORMAL = 1
    HALTED = 2
    CANCEL_ONLY = 3


class HaltReason:
    EMERGENCY_MAINTENANCE = 2
    SCHEDULED_MAINTENANCE = 3



API_URL = 'https://www.okex.com'

CONTENT_TYPE = 'Content-Type'
OK_ACCESS_KEY = 'OK-ACCESS-KEY'
OK_ACCESS_SIGN = 'OK-ACCESS-SIGN'
OK_ACCESS_TIMESTAMP = 'OK-ACCESS-TIMESTAMP'
OK_ACCESS_PASSPHRASE = 'OK-ACCESS-PASSPHRASE'


ACEEPT = 'Accept'
COOKIE = 'Cookie'
LOCALE = 'Locale='

APPLICATION_JSON = 'application/json'

GET = "GET"
POST = "POST"
DELETE = "DELETE"

SERVER_TIMESTAMP_URL = '/api/general/v5/time'

# account
POSITION_RISK = '/api/v5/account/account-position-risk'
BALANCE = '/api/v5/account/balance'
POSITIONS = '/api/v5/account/positions'
BILLS = '/api/v5/account/bills'
BILLS_ARCHIVE = '/api/v5/account/bills-archive'
CONFIG = '/api/v5/account/config'


# asset
DEPOSIT_ADDRESS = '/api/v5/asset/deposit-address'
BALANCES = '/api/v5/asset/balances'


# market
TICKERS = '/api/v5/market/tickers'
TICKER = '/api/v5/market/ticker'
INDEX_TICKERS = '/api/v5/market/index-tickers'
BOOKS = '/api/v5/market/books'
CANDLES = '/api/v5/market/candles'
HISTORY_CANDLES = '/api/v5/market/history-candles'

# public
INSTRUMENTS = '/api/v5/public/instruments'


# trade
ORDER = '/api/v5/trade/order'
BATCH_ORDERS = '/api/v5/trade/batch-orders'
CANCEL_ORDER = '/api/v5/trade/cancel-order'
CANCEL_BATCH_ORDERS = '/api/v5/trade/cancel-batch-orders'


# #############
# http header
API_URL = 'https://www.okex.com'

CONTENT_TYPE = 'Content-Type'
OK_ACCESS_KEY = 'OK-ACCESS-KEY'
OK_ACCESS_SIGN = 'OK-ACCESS-SIGN'
OK_ACCESS_TIMESTAMP = 'OK-ACCESS-TIMESTAMP'
OK_ACCESS_PASSPHRASE = 'OK-ACCESS-PASSPHRASE'

ACEEPT = 'Accept'
COOKIE = 'Cookie'
LOCALE = 'Locale='

APPLICATION_JSON = 'application/json'

GET = "GET"
POST = "POST"

SERVER_TIMESTAMP_URL = '/api/v5/public/time'

# account
POSITION_RISK = '/api/v5/account/account-position-risk'
ACCOUNT_INFO = '/api/v5/account/balance'
POSITION_INFO = '/api/v5/account/positions'
BILLS_DETAIL = '/api/v5/account/bills'
BILLS_ARCHIVE = '/api/v5/account/bills-archive'
ACCOUNT_CONFIG = '/api/v5/account/config'
POSITION_MODE = '/api/v5/account/set-position-mode'
SET_LEVERAGE = '/api/v5/account/set-leverage'
MAX_TRADE_SIZE = '/api/v5/account/max-size'
MAX_AVAIL_SIZE = '/api/v5/account/max-avail-size'
ADJUSTMENT_MARGIN = '/api/v5/account/position/margin-balance'
GET_LEVERAGE = '/api/v5/account/leverage-info'
MAX_LOAN = '/api/v5/account/max-loan'
FEE_RATES = '/api/v5/account/trade-fee'
INTEREST_ACCRUED = '/api/v5/account/interest-accrued'
INTEREST_RATE = '/api/v5/account/interest-rate'
SET_GREEKS = '/api/v5/account/set-greeks'
MAX_WITHDRAWAL = '/api/v5/account/max-withdrawal'

# funding
DEPOSIT_ADDRESS = '/api/v5/asset/deposit-address'
GET_BALANCES = '/api/v5/asset/balances'
FUNDS_TRANSFER = '/api/v5/asset/transfer'
WITHDRAWAL_COIN = '/api/v5/asset/withdrawal'
DEPOSIT_HISTORIY = '/api/v5/asset/deposit-history'
WITHDRAWAL_HISTORIY = '/api/v5/asset/withdrawal-history'
CURRENCY_INFO = '/api/v5/asset/currencies'
PURCHASE_REDEMPT = '/api/v5/asset/purchase_redempt'
BILLS_INFO = '/api/v5/asset/bills'

# Market Data
TICKERS_INFO = '/api/v5/market/tickers'
TICKER_INFO = '/api/v5/market/ticker'
INDEX_TICKERS = '/api/v5/market/index-tickers'
ORDER_BOOKS = '/api/v5/market/books'
MARKET_CANDLES = '/api/v5/market/candles'
HISTORY_CANDLES = '/api/v5/market/history-candles'
INDEX_CANSLES = '/api/v5/market/index-candles'
MARKPRICE_CANDLES = '/api/v5/market/mark-price-candles'
MARKET_TRADES = '/api/v5/market/trades'
VOLUMNE = '/api/v5/market/platform-24-volume'
ORACLE = '/api/v5/market/oracle'
TIER = '/api/v5/public/tier'

# Public Data
INSTRUMENT_INFO = '/api/v5/public/instruments'
DELIVERY_EXERCISE = '/api/v5/public/delivery-exercise-history'
OPEN_INTEREST = '/api/v5/public/open-interest'
FUNDING_RATE = '/api/v5/public/funding-rate'
FUNDING_RATE_HISTORY = '/api/v5/public/funding-rate-history'
PRICE_LIMIT = '/api/v5/public/price-limit'
OPT_SUMMARY = '/api/v5/public/opt-summary'
ESTIMATED_PRICE = '/api/v5/public/estimated-price'
DICCOUNT_INTETEST_INFO = '/api/v5/public/discount-rate-interest-free-quota'
SYSTEM_TIME = '/api/v5/public/time'
LIQUIDATION_ORDERS = '/api/v5/public/liquidation-orders'
MARK_PRICE = '/api/v5/public/mark-price'

# TRADE
PLACR_ORDER = '/api/v5/trade/order'
BATCH_ORDERS = '/api/v5/trade/batch-orders'
CANAEL_ORDER = '/api/v5/trade/cancel-order'
CANAEL_BATCH_ORDERS = '/api/v5/trade/cancel-batch-orders'
AMEND_ORDER = '/api/v5/trade/amend-order'
AMEND_BATCH_ORDER = '/api/v5/trade/amend-batch-orders'
CLOSE_POSITION = '/api/v5/trade/close-position'
ORDER_INFO = '/api/v5/trade/order'
ORDERS_PENDING = '/api/v5/trade/orders-pending'
ORDERS_HISTORY = '/api/v5/trade/orders-history'
ORDERS_HISTORY_ARCHIVE = '/api/v5/trade/orders-history-archive'
ORDER_FILLS = '/api/v5/trade/fills'
PLACE_ALGO_ORDER = '/api/v5/trade/order-algo'
CANCEL_ALGOS = '/api/v5/trade/cancel-algos'
ORDERS_ALGO_OENDING = '/api/v5/trade/orders-algo-pending'
ORDERS_ALGO_HISTORY = '/api/v5/trade/orders-algo-history'

# SubAccount
BALANCE = '/api/v5/account/subaccount/balances'
BILLs = '/api/v5/asset/subaccount/bills'
DELETE = '/api/v5/users/subaccount/delete-apikey'
RESET = '/api/v5/users/subaccount/modify-apikey'
CREATE = '/api/v5/users/subaccount/apikey'
VIEW_LIST = '/api/v5/users/subaccount/list'
CONTROL_TRANSFER = '/api/v5/asset/subaccount/transfer'

# status
STATUS = '/api/v5/system/status'



import asyncio
import json

import aiohttp
from notecoin.huobi.connection import (RestApiSyncClient, SubscribeClient,
                                       WebSocketReqClient)
from notecoin.huobi.constant import (AccountBalanceMode, AccountType,
                                     HttpMethod, TransferFuturesPro,
                                     TransferMasterType,
                                     get_default_server_url)
from notecoin.huobi.utils import get_current_timestamp
from notecoin.huobi.utils.input_checker import (check_currency, check_in_list,
                                                check_should_not_none)


class AccountClient(object):

    def __init__(self, *args, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: Init logger, default is False, True will init logger handler
        """
        self.__kwargs = kwargs
        self.rest_api_sync_client = RestApiSyncClient(*args, **kwargs)
        self.web_socket_req_client = WebSocketReqClient(*args, **kwargs)
        self.sub_socket_req_client = SubscribeClient(*args, **kwargs)

    def get_accounts(self):
        """
        Get the account list.
        :return: The list of accounts data.
        """
        channel = "/v1/account/accounts"
        return self.rest_api_sync_client.request_process(HttpMethod.GET_SIGN, channel, {})

    def get_balance(self, account_id: int):
        """
        Get the account list.
        :return: The list of accounts data.
        """
        check_should_not_none(account_id, "account-id")
        params = {
            "account-id": account_id
        }

        def get_channel():
            path = "/v1/account/accounts/{}/balance"
            return path.format(account_id)

        return self.rest_api_sync_client.request_process(HttpMethod.GET_SIGN, get_channel(), params)

    def get_account_by_type_and_symbol(self, account_type, symbol):
        accounts = self.get_accounts()
        if accounts and len(accounts):
            for account_obj in accounts:
                if account_obj.type == account_type:
                    if account_type == AccountType.MARGIN:
                        if symbol == account_obj.subtype:
                            return account_obj
                    else:
                        return account_obj

        return None

    @staticmethod
    async def async_get_account_balance(balance_full_url, account_id, ret_map):
        async with aiohttp.ClientSession() as session:
            async with session.get(balance_full_url) as resp:
                _json = await resp.json()
                ret_map[account_id] = _json
                return _json

    """
    (SDK encapsulated api) to easily use but not recommend for low performance and frequence limitation
    """

    def get_account_balance(self) -> list:
        """
        Get the balance of a all accounts.

        :return: The information of all account balance.
        """
        server_url = get_default_server_url(self.__kwargs.get("url"))
        tasks = []
        account_obj_map = {}
        accounts = self.get_accounts()
        account_balance_list = []
        account_balance_json_map = {}
        for account_item in accounts:
            account_obj_map[account_item.id] = account_item
            params = {"account-id": account_item.id}

            def get_channel():
                path = "/v1/account/accounts/{}/balance"
                return path.format(account_item.id)

            balance_request = self.rest_api_sync_client.create_request(HttpMethod.GET_SIGN, get_channel(), params)

            balance_url = server_url + balance_request.url
            tasks.append(asyncio.ensure_future(
                self.async_get_account_balance(balance_url, account_item.id, account_balance_json_map)))

        loop = asyncio.get_event_loop()
        try:
            loop.run_until_complete(asyncio.wait(tasks))
        except Exception as ee:
            print(ee)
        finally:
            # loop.close()  #for thread safe, the event loop can't be closed
            pass

        for account_id, account_balance_json in account_balance_json_map.items():
            account_balance_list.append(account_balance_json.get("data", {}))

        del account_balance_json_map
        del tasks
        return account_balance_list

    def get_account_balance_by_subuid(self, sub_uid):
        """
        Get account balance of a sub-account.

        :param sub_uid: the specified sub account id to get balance for.
        :return: the balance of a sub-account specified by sub-account uid.
        """
        check_should_not_none(sub_uid, "sub-uid")
        params = {
            "sub-uid": sub_uid
        }

        def get_channel():
            path = "/v1/account/accounts/{}"
            return path.format(sub_uid)

        return self.rest_api_sync_client.request_process(HttpMethod.GET_SIGN, get_channel(), params)

    def get_aggregated_subuser_balance(self):
        """
        Get the aggregated balance of all sub-accounts of the current user.

        :return: The balance of all the sub-account aggregated.
        """
        params = {}
        channel = "/v1/subuser/aggregate-balance"
        return self.rest_api_sync_client.request_process(HttpMethod.GET_SIGN, channel, params)

    def transfer_between_parent_and_subuser(self, sub_uid: int, currency: str, amount: float,
                                            transfer_type: TransferMasterType):
        """
        Transfer Asset between Parent and Sub Account.

        :param sub_uid: The target sub account uid to transfer to or from. (mandatory)
        :param currency: The crypto currency to transfer. (mandatory)
        :param amount: The amount of asset to transfer. (mandatory)
        :param transfer_type: The type of transfer, see {@link TransferMasterType} (mandatory)
        :return: The order id.
        """
        check_currency(currency)
        check_should_not_none(sub_uid, "sub-uid")
        check_should_not_none(amount, "amount")
        check_should_not_none(transfer_type, "type")

        params = {
            "sub-uid": sub_uid,
            "currency": currency,
            "amount": amount,
            "type": transfer_type
        }
        channel = "/v1/subuser/transfer"

        return self.rest_api_sync_client.request_process(HttpMethod.POST_SIGN, channel, params)

    def sub_account_update(self, mode: AccountBalanceMode, callback, error_handler=None):
        """
        Subscribe accounts update

        :param mode: subscribe mode
                "0" : for balance
                "1" : for available and balance
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
                              between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """

        check_should_not_none(callback, "callback")
        if str(mode) == AccountBalanceMode.TOTAL:
            mode = AccountBalanceMode.TOTAL
        else:
            mode = AccountBalanceMode.BALANCE

        def accounts_update_channel(_mode=0):
            channel = dict()
            channel["action"] = "sub"
            if _mode is None:
                channel["ch"] = "accounts.update"
            else:
                channel["ch"] = "accounts.update#{mode}".format(mode=_mode)
            return json.dumps(channel)

        def subscription(connection):
            connection.send(accounts_update_channel(mode))

        self.sub_socket_req_client.execute_subscribe_v2(subscription, callback, error_handler, is_trade=True)

    def req_account_balance(self, callback, client_req_id=None, error_handler=None):
        """
        Subscribe account changing event. If the balance is updated,
        server will send the data to client and onReceive in callback will be called.

        :param client_req_id: client request ID
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(account_event: 'AccountEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
                              between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """

        check_should_not_none(callback, "callback")

        def request_account_list_channel(_client_req_id=None):
            channel = dict()
            channel["op"] = "req"
            channel["topic"] = "accounts.list"
            channel["cid"] = str(_client_req_id) if _client_req_id else str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            connection.send(request_account_list_channel(client_req_id))

        self.web_socket_req_client.execute_subscribe_v1(subscription, callback, error_handler, is_trade=True)

    def transfer_between_futures_and_pro(self, currency: str, amount: float,
                                         transfer_type: TransferFuturesPro) -> int:
        """
        Transfer Asset between Futures and Contract.

        :param currency: The crypto currency to transfer. (mandatory)
        :param amount: The amount of asset to transfer. (mandatory)
        :param transfer_type: The type of transfer, need be "futures-to-pro" or "pro-to-futures" (mandatory)
        :return: The order id.
        """

        check_currency(currency)
        check_should_not_none(currency, "currency")
        check_should_not_none(amount, "amount")
        check_should_not_none(transfer_type, "transfer_type")
        params = {
            "currency": currency,
            "amount": amount,
            "type": transfer_type
        }
        channel = "/v1/futures/transfer"

        return self.rest_api_sync_client.request_process(HttpMethod.POST_SIGN, channel, params)

    def get_account_history(self, account_id: int, currency: str = None,
                            transact_types: str = None, start_time: int = None, end_time: int = None,
                            sort: str = None, size: int = None):
        """
        get account change record
        :param account_id: account id (mandatory)
        :param currency: currency as "btc,eth" (optional)
        :param transact_types: see AccountTransactType, the value can be
                              "trade" (交易),"etf"（ETF申购）, "transact-fee"（交易手续费）,
                              "deduction"（手续费抵扣）, "transfer"（划转）, "credit"（借币）,
                              "liquidation"（清仓）, "interest"（币息）, "deposit"（充币），
                              "withdraw"（提币）, "withdraw-fee"（提币手续费）, "exchange"（兑换）,
                              "other-types"（其他） (optional)
        :param start_time
        :param end_time: for time range to search (optional)
        :param sort: see SortDesc, "asc" or "desc" (optional)
        :param size: page size (optional)

        :return: account change record list.
        """
        check_should_not_none(account_id, "account-id")
        params = {
            "account-id": account_id,
            "currency": currency,
            "transact-types": transact_types,
            "start-time": start_time,
            "end-time": end_time,
            "sort": sort,
            "size": size
        }
        channel = "/v1/account/history"
        return self.rest_api_sync_client.request_process(HttpMethod.GET_SIGN, channel, params)

    def post_sub_uid_management(self, sub_uid: int, action: str):
        """
        use to freeze or unfreeze the sub uid

        :return: user and status.
        """

        check_should_not_none(sub_uid, "subUid")
        check_should_not_none(action, "action")

        params = {
            "subUid": sub_uid,
            "action": action
        }
        channel = "/v2/sub-user/management"
        return self.rest_api_sync_client.request_process(HttpMethod.POST_SIGN, channel, params)

    def get_account_ledger(self, account_id: int, currency: str = None, transact_types: str = None,
                           start_time: int = None, end_time: int = None, sort: str = None, limit: int = None,
                           from_id: int = None) -> list:
        """
        get account ledger
        :param from_id:  from_id:
        :param account_id: account id (mandatory)
        :param currency: currency as "btc,eth" (optional)
        :param transact_types: see AccountTransactType, the value can be "trade" (交易),"etf"（ETF申购）,
                                "transact-fee"（交易手续费）, "deduction"（手续费抵扣）, "transfer"（划转）,
                                "credit"（借币）, "liquidation"（清仓）, "interest"（币息）,
                                "deposit"（充币），"withdraw"（提币）, "withdraw-fee"（提币手续费）,
                                "exchange"（兑换）, "other-types"（其他） (optional)
        :param start_time
        :param end_time: for time range to search (optional)
        :param sort: see SortDesc, "asc" or "desc" (optional)
        :param limit: page size (optional)
        :return: account ledger list.
        """

        check_should_not_none(account_id, "accountId")

        params = {
            "accountId": account_id,
            "currency": currency,
            "transactTypes": transact_types,
            "startTime": start_time,
            "endTime": end_time,
            "sort": sort,
            "limit": limit,
            "fromId": from_id
        }
        channel = "/v2/account/ledger"
        return self.rest_api_sync_client.request_process(HttpMethod.GET_SIGN, channel, params)

    def post_account_transfer(self, from_user: int, from_account_type: str, from_account: int, to_user: int,
                              to_account_type: str, to_account: int, currency: str, amount: str):
        check_should_not_none(from_user, "from-user")
        check_should_not_none(from_account_type, "from-account-type")
        check_should_not_none(from_account, "from_account")
        check_should_not_none(to_user, "to-user")
        check_should_not_none(to_account, "to-account")
        check_should_not_none(to_account_type, "to-account")
        check_should_not_none(currency, "currency")

        check_in_list(from_account_type, [AccountType.SPOT], "from_account_type")
        check_in_list(to_account_type, [AccountType.SPOT], "to_account_type")

        params = {
            "from-user": from_user,
            "from-account-type": from_account_type,
            "from-account": from_account,
            "to-user": to_user,
            "to-account-type": to_account_type,
            "to-account": to_account,
            "currency": currency,
            "amount": amount
        }
        channel = "/v1/account/transfer"
        return self.rest_api_sync_client.request_process(HttpMethod.POST_SIGN, channel, params)

    def get_account_asset_valuation(self, account_type, valuation_currency: str = None, sub_uid: str = None):
        check_should_not_none(account_type, "account-type")

        params = {
            "accountType": account_type,
            "valuationCurrency": valuation_currency.upper(),
            "subUid": sub_uid
        }
        channel = "/v2/account/asset-valuation"

        return self.rest_api_sync_client.request_process(HttpMethod.GET_SIGN, channel, params)

    def get_account_point(self, sub_uid: str = None):
        params = {
            "subUid": sub_uid
        }
        channel = "/v2/point/account"

        return self.rest_api_sync_client.request_process(HttpMethod.GET_SIGN, channel, params)

    def post_point_transfer(self, from_uid: str, to_uid: str, group_id: str, amount: str):
        channel = "/v2/point/transfer"
        params = {
            "fromUid": from_uid,
            "toUid": to_uid,
            "groupId": group_id,
            "amount": amount
        }

        return self.rest_api_sync_client.request_process(HttpMethod.POST_SIGN, channel, params)



from notecoin.strategy.utils import trade_api


class OkexCoin:
    def __init__(self, coin_id='METIS-USDT', price=0, count=0.):
        self.coin_id = coin_id
        self.count = count
        self.price_in = price
        self.price = self.price_in

    @property
    def money(self):
        return self.price * self.count

    def buy(self):
        return trade_api.place_order(instId=self.coin_id, tdMode='cash', side='buy', ordType='market', sz='50')

    def sell(self):
        return trade_api.place_order(instId=self.coin_id, tdMode='cash', side='sell', ordType='market', sz=self.count)

    @staticmethod
    def instance_by_account(data):
        okex = OkexCoin(coin_id=f"{data['ccy']}-USDT", count=float(data['availBal']),
                        price=float(data['eqUsd']) / float(data['availBal']))
        return okex

    @staticmethod
    def instance_by_new(coin_id):
        okex = OkexCoin(coin_id=coin_id)
        okex.buy()
        return okex

    def watch(self):
        money = self.money
        if money > 51:
            self.sell()

    def __str__(self):
        return f"{self.coin_id}\t{self.count}\t{self.price_in}\t{self.price}\t{self.money}"

    def to_json(self):
        return {
            "coin_id": self.coin_id,
            # "count": self.count,
            # "price_in": self.price_in,
            # "price": price,
            "usdt": round(self.count * self.price, 2)
        }



from notecoin.database.base import Base, BaseTable, create_all, session
from notecoin.okex.client.const import account_api
from sqlalchemy import Column, Float, String


class OkexClientAccountBalance(Base, BaseTable):
    __tablename__ = 'okex_client_account_balance'
    ccy = Column(String(120), comment='币种', primary_key=True)
    eq = Column(Float(), comment='币种总权益', primary_key=True)
    cashBal = Column(Float(), comment='币种余额')
    uTime = Column(Float(), comment='币种余额信息的更新时间毫秒')

    disEq = Column(Float(), comment='美金层面币种折算权益')
    availBal = Column(Float(), comment='可用余额')
    frozenBal = Column(Float(), comment='币种占用金额')
    ordFrozen = Column(Float(), comment='挂单冻结数量')
    twap = Column(Float(), comment='当前负债币种触发系统自动换币的风险')
    eqUsd = Column(Float(), comment='币种权益美金价值')
    stgyEq = Column(Float(), comment='策略权益')

    def __init__(self, *args, **kwargs):
        self.ccy = kwargs.get("ccy")
        self.eq = kwargs.get("eq", 0)
        self.cashBal = kwargs.get("cashBal", 0)
        self.uTime = kwargs.get("uTime", 0)
        self.disEq = kwargs.get("disEq", 0)
        self.availBal = kwargs.get("availBal", 0)
        self.frozenBal = kwargs.get("frozenBal", 0)
        self.ordFrozen = kwargs.get("ordFrozen", 0)

        self.twap = kwargs.get("twap", 0)
        self.eqUsd = kwargs.get("eqUsd", 0)
        self.stgyEq = kwargs.get("stgyEq", 0)

    def __repr__(self):
        return f"ccy{self.ccy},availBal:{self.availBal}"

    @staticmethod
    def update():
        
        data = account_api.get_account().data[0]['details']
        create_all()

        for detail in data:
            session.merge(OkexClientAccountBalance(**detail))
        session.commit()



from notecoin.database.base import Base
from sqlalchemy import BIGINT, Column, Float, String


class BaseTable(object):
    def json(self):
        res = {k: v for k, v in self.__dict__.items() if not k.startswith("_")}
        return res


class OkexSocketPublicTickers(Base, BaseTable):
    __tablename__ = 'okex_socket_public_tickets'
    instType = Column(String(120), comment='产品类型', primary_key=True)
    instId = Column(String(120), comment='产品ID', primary_key=True)

    last = Column(Float(), comment='最新成交价')
    lastSz = Column(Float(), comment='最新成交的数量')

    askPx = Column(Float(), comment='卖一价')
    askSz = Column(Float(), comment='卖一价对应的量')
    bidPx = Column(Float(), comment='买一价')
    bidSz = Column(Float(), comment='买一价对应的量')

    open24h = Column(Float(), comment='24小时开盘价')
    high24h = Column(Float(), comment='24小时最高价')
    low24h = Column(Float(), comment='24小时最低价')
    volCcy24h = Column(Float(), comment='24小时成交量，以计价货币为单位')
    vol24h = Column(Float(), comment='24小时成交量，以交易货币为单位')

    sodUtc0 = Column(Float(), comment='UTC 0时开盘价')
    sodUtc8 = Column(Float(), comment='UTC+8时开盘价')

    ts = Column(BIGINT(), comment='数据产生时间，Unix时间戳的毫秒数格式')

    def __init__(self, *args, **kwargs):
        self.instType = kwargs.get("instType", "SPOT")
        self.instId = kwargs.get("instId", "")
        self.last = kwargs.get("last", 0)
        self.lastSz = kwargs.get("lastSz", 0)
        self.askPx = kwargs.get("askPx", 0)
        self.askSz = kwargs.get("askSz", 0)
        self.bidPx = kwargs.get("bidPx", 0)

        self.bidSz = kwargs.get("bidSz", 0)
        self.open24h = kwargs.get("open24h", 0)
        self.high24h = kwargs.get("high24h", 0)
        self.low24h = kwargs.get("low24h", 0)
        self.volCcy24h = kwargs.get("volCcy24h", 0)

        self.vol24h = kwargs.get("vol24h", 0)
        self.sodUtc0 = kwargs.get("sodUtc0", 0)
        self.sodUtc8 = kwargs.get("sodUtc8", 0)
        self.ts = kwargs.get("ts", 0)

    def __repr__(self):
        return f"instType{self.instType},instId:{self.instId}"
